/* BNS 512 SLM driver for linux (c) Matt Clark 2001 */

/*	(host)			 (card)			     (port)			*
 *      |---------|									*
 *      | HOST    |                                          |----------|		*
 *      |         |<=====PCI====>|-----------|<====cable====>|Dual port |		*
 *      |---------|              | PCI card  |               |ext. card |		*
 *   |---------------|           |---|||||||||               |----------|		*
 *   |---------------|            ^                              |			*
 *                                |                              |			*
 *                                v                              v			*
 *                               external IOs                   |=|			*
 *                                                              SLM (slm)		*
 *											*/


/* where BNS refer to SDRAM it looks like this is ram on the PCI card ("Card memory")	*/
/* where BNS refer to SRAM or Dual-Port SRAM it looks like the 1 frame			*
 * buffer in the external card ("port memory") 						*/

/************************ semantics note ************************************************
 * in order to not get mixed up 							*
 * image-> image period -> length of time in frame periods an image is displayed	*
 * image-> image(s) -> image memory (on card)						*
 * frame-> frame period->a time slot when the image is transferred from (port) to (slm)	*/

/************************ DRIVER / CARD DETAILS *****************************************/
/* because BNS have not given their system a unique ID we have to be careful we don't 	*
 * encounter anything else with this ID- matt 						*/


/* essential include here 								*/
#include <linux/ioctl.h>
#include <asm/types.h>
#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif

/* we don't export anything */
//#define EXPORT_NO_SYMBOLS
/******************** HACKS / TWEAKS / OPTIONS *****************************************/

/* define this to test the SDRAM controllers before each write
* #define BNS_DODGY_SDRAM
*
* define this to lock out writes during dual port transfers
* #define BNS_2PORT_LOCK_OUT
*
* define this to enable SDRAM resets on enabled boards
* #define BNS_SDRAM_RESET_ENABLE
*
* define this to ignore 0x89 errors on SDRAM 2
* #define BNS_0X89_SDRAM_ERROR
*/

/* define this to enable profiling information to be monitored */
#define BNS_PROFILE

/* leave this whatever....*/
#ifdef BNS_PROFILE
#define TIC(a)	rdtscl(a)
#else
#define TIC(a)
#endif










/************************* DRIVER IOCTL DEFINES	*****************************************/
/* this magic number has been chosen as the next in our sequence of 2.4 drivers- it may	*
 * clash with other magic numbers feel free to change (but recompile everything		*
 * inlcuding ALL user code that accesses the device driver				*/
/* where data flow is indicated the size argument is 0 indicating call by value or 	*
 * variable size transfer								*/

#define BNS_MAGIC		105

/* make it possible to force and unload during development 				*/
#define	BNS_HARDRESET		_IO(BNS_MAGIC,0)

/* send frame x to the slm / make frame x the next upload				*/
#define BNS_DISPLAY_IMAGE	_IOW(BNS_MAGIC,1,0)	/* pass by value		*/
#define BNS_UPLOAD_IMAGE	_IOW(BNS_MAGIC,2,0)	/* pass by value		*/
#define BNS_SET_ROW		_IOW(BNS_MAGIC,3,0)	/* pass by value		*/

/* turn the system on / off								*/
#define BNS_POWER_ON		_IO(BNS_MAGIC,4)
#define BNS_POWER_OFF		_IO(BNS_MAGIC,5)


/* Now I can envisage keeping a frame rotation list in the driver and using the 	*
 * interrupts to service stepping throught the list					*/

#define BNS_SET_LIST_LENGTH	_IOW(BNS_MAGIC,7,0)	/* pass by value		*/
#define BNS_SET_LIST		_IOW(BNS_MAGIC,8,0)	/* pass array by pointer	*/

#define BNS_START_LIST		_IOW(BNS_MAGIC,9,0)	/* pass by value		*/
#define BNS_START		0			/* same as zero, start pos	*/
#define BNS_STOP_LIST		_IO(BNS_MAGIC,10)

/****************************************************************************************/
/* since the DPEN issue is dues to crappy hw design and we can't side step it change	*/
/* driver design to force the user to explicitly disable DPEN (as this is expensive)	*/
/* before writing-	matt nov 2001							*/

#define BNS_ENABLE_WRITE	_IO(BNS_MAGIC,20)	/* enabling writing / DPEN 0	*/
#define BNS_DISBALE_DPEN	_IO(BNS_MAGIC,20)	/* ditto			*/
#define BNS_DISABLE_WRITE	_IO(BNS_MAGIC,21)	/* disbale writes- 		*/
#define BNS_ENABLE_DPEN		_IO(BNS_MAGIC,21)	/* ditto			*/

#define BNS_SET_DPEN_MODE	_IOW(BNS_MAGIC,25,0)	/* debug for setting mode	*/
#define BNS_DPEN_MODE_BUSY	0			/* use busy waits		*/
#define BNS_DPEN_MODE_SLEEP	1			/* use schedule			*/
#define BNS_DPEN_MODE_PROBE	2			/* probe- not implemented in hw */
#define BNS_DPEN_MODE_MAX	BNS_DPEN_MODE_PROBE	/* limit on options 		*/

#define BNS_SET_DPEN_DELAY	_IOW(BNS_MAGIC,26,0)	/* debug for setting dpen delay	*/
/* set the default delay, according to kipp 4*164 is good but we saw 1:10000 locking up */
/* could be due to udelay accuracy? anyway exclude 6 frame periods+PARANOIA just to be	*/
/* safe (1ms)!										*/
#define DPEN_PARANOIA		16
#define BNS_DEFAULT_DPEN_DELAY	(164*6)+DPEN_PARANOIA	/* default DPEN delay		*/



/* ioctl bns set parameters start at 50							*/
#define BNS_SET_LASERA		_IOW(BNS_MAGIC,50,0)	/* pass by value		*/
#define BNS_SET_LASERB		_IOW(BNS_MAGIC,51,0)	/* pass by value		*/
#define BNS_SET_PERIODS		_IOW(BNS_MAGIC,52,0)	/* pass by value		*/

/* ioctl info requests start with 100							*/
#define BNS_GET_IMAGE		_IOR(BNS_MAGIC,100,0)
#define BNS_GET_MAX_IMAGE	_IOR(BNS_MAGIC,101,0)

#define GIGAHERTZ			1e9
#define BNS_SLM_CLOCK			50e6
#define BNS_CARD_CLOCK			25e6
#define BNS_SLM_NS			GIGAHERTZ/BN_SLM_CLOCK
#define BNS_CARD_NS			GIGAHERTZ/BN_CARD_CLOCK
#define BNS_SLM_IMAGE_TICKS		8192+16	/* 8192 clks transfer +16 clks pipeline	*/
#define BNS_SLM_TRANSFER		32
/* a bns frame period									*/
#define BNS_SLM_FP			BNS_SLM_IMAGE_TICKS*BNS_SLM_NS
/* minimum number of frame periods- 4 for image load to slm and 2 for +&- images 	*/
/* what happens during load ?								*/
#define BNS_MIN_FP			6


#define BNS_SLM_BW_PER_FP		BNS_SLM_IMAGE_TICKS*BNS_SLM_TRANSFER /*512^2*/





#define BNS_FRAME_RATE		50e6
#define BNS_FRAME_PERIOD	1/BNS_FRAME_RATE
#define BNS_FRAME_NS		1000000000/BNS_FRAME_RATE
#define BNS_IMAGE_LOAD_FP	6
#define BNS_IMAGE_LOAD_TIME	BNS_FRAME_PERIOD*BNS_IMAGE_LOAD_FP
#define BNS_IMAGE_LOAD_NS	BNS_FRAME_NS*BNS_IMAGE_LOAD_FP

/* define default behaviour, slm off- inactive, slm_list go thru list via interrupts	*
 * slm_image display a single image							*/

#define BNS_SLM_OFF			0
#define BNS_SLM_LIST		1
#define BNS_SLM_IMAGE		2
#define BNS_SLM_NO_IMAGE		-1		
#define BNS_MAX_ROWS		512
#define BNS_ROW_SIZE		512
#define BNS_N_ROWS		512

#define PERFORM_SCRAMBLE	0
#define RAW_TRANSFER		1

#define BNS_POWER_IS_OFF	0x00
#define BNS_POWER_DAC		0x01
#define BNS_POWER_OPAMP		0x02
#define BNS_POWER_SLM		0x04
#define BNS_POWER_VOK		0x08

/* Frame control list									*/
typedef struct BNS_irl_object{
	__u32	image;
	__u32	periods;
	} BNS_irl_object;

/***** EXCLUDE THE LINES BELOW FROM USER CODE (don't bother to indent thought) **********/
#ifdef MODULE
//MODULE_LICENSE("GPL") 

#define SLM_MOD_NAME	"slm"
#define IRQ_ID		"BNS-slm"

#ifdef SLM_DEBUG
#define PRINT_DEBUG(a,args...)  printk(a,## args); 
#else 
#define PRINT_DEBUG(a,args...)
#endif



/****************** PCI CARD DEFINES ****************************************************/
/* note: BNS don't have their own vendor id and are using PLX's as of 22 oct 2001    	*/
#define BNS_VENDOR_ID		0x10b5		/* BNS=0xb510 */
#define BNS_SUB_VENDOR_ID	0x10b5		/* BNS=0xb510 */

/* note: 0x0100 is the PLX generic ISA device ID as of 22 oct 2001			*/
#define BNS_DEVICE_ID		0x9050
#define BNS_SUB_DEVICE_ID	0x9050

/* the pci regions on the card that give the io and base memory addresses		*/
/* we don't have these yet - matt oct 2001						*/
#define BNS_SLM_PLX		0	/* just an educated guess- matt oct 2001	*/
#define BNS_SLM_IO		2
#define BNS_SLM_MEM		3

/* plx region is 128, io region is 64 bytes and mem region is 256k			*/
#define BNS_SLM_PLX_SIZE	0x00000080
#define BNS_SLM_IO_SIZE		0x00000040
#define BNS_SLM_MEM_SIZE	0x00040000



/****************************************************************************************/

/******************************** CARD MEMORY ARCHITECTURE *****************************/
/* the card memory does not map directly onto the SLM memory- this is a right pain in 	*
 * the arse as it invloves calulating the correct mapping on the fly or preparing the 	*
 * image data in a scrambled format- THIS WOULD BE BETTER DONE IN THE HARDWARE- matt 	*/
/* the mapping goes like this the full address is 18 bits long, first 9 bits the column *
 * the next 9 bits the row.  the ordering within a row (col) is not linear and follows  *
 * rule col=(add&111000000)+(add&000111100)>2+(add&000000011)<4				*
 * ie the first two bits are shifted to bits 4+5 and the next four bits are shifted to	*
 * bits 0-3.										*
 *	* I suggest doing this in hardware, either by making the writes row at a time 	*
 *	* and linear address as seen by the SLM (requires a row register) and 		*
 *	* hardware translaing the address row by row					*
 *	* or										*
 *	* sacrificing one of the images for permanent IO, the idea is you always write  *
 *	* to one image (linear to the slm)- this is then transferred to one of the	*
 *	* other images and transposed correctly on the way- you would los one image	*
 *	* but the transposing is really easy in hardware 				*
 *	* Why bother?									*
 *	* There is lots of image preparation hardware that knocks the spots off of CPU  *
 *	* performance let this prepare the data and blit it to the card- however this   *
 *	* requires that the card can accept the data as a byte map unscramble		*
 *	* it also takes the CPU out of the loop and simplifies the harware memory 	*
 *	* interace.  This is essentially a no-op which could be achieved by wiring.	*/

/* address in binary 		x8 x7 x6 x5 x4 x3 x2 x1 x0				*
 *				|  |  |  |        |  |  |
 *				|  |  |   \        \/  /				*
 *				|  |  |    \       /\ /					*
 *				|  |  |     \     /  X					*
 *				|  |  |      \   /  / \					*
 *				|  |  |       \ /  /   \				*
 *				|  |  |        /  /     |				*
 *				|  |  |       /| /      |				*
 *				|  |  |      / |/       |				*
 *				|  |  |     /  /        |				*
 *				|  |  |    /  /|        |				*
 *				|  |  |   /  / |        |				*
 *				|  |  |  |  |  |        |
 * address in slm column	x8 x7 x6 x1 x0 x5 x4 x3 x2				*/
 
/* conversion constants for physical address (card) address -> column on slm		*/
#define BNS_ADD_ROW             0x0003ffc0 
#define BNS_ADD_BANK		0x000000c0 	/* not needed */
#define BNS_BANK_SHIFT		6		/* not needed */
#define BNS_ADD_BYTES		0x0000003c
#define BNS_ADD_STRIP		0x00000003

/* conversion constants for column on slm -> physical address (card) address		*/
#define BNS_SLM_BYTES		0x0000000f
#define BNS_SLM_STRIP		0x00000030

/* how much they are shifted								*/
#define BNS_BYTE_SHIFT		2
#define BNS_STRIP_SHIFT		4

/* note the extra brackets here- they are required if the arg contains operations	*/
/* macro to convert from card address to slm column					*/
#define BNS_ADD_TO_SLM(a)	((a)&BNS_ADD_ROW)+(((a)&BNS_ADD_BYTES)>>BNS_BYTE_SHIFT)+(((a)&BNS_ADD_STRIP)<<BNS_STRIP_SHIFT)

/* macro to convert from slm _column to card address					*/
#define BNS_SLM_TO_ADD(c)	((c)&BNS_ADD_ROW)+(((c)&BNS_SLM_BYTES)<<BNS_BYTE_SHIFT)+(((c)&BNS_SLM_STRIP)>>BNS_STRIP_SHIFT)

/****************************************************************************************/


/*************************PLX CARD REGISTER MAP *****************************************/
/* Register map working from the PLX 9052 data book v 2.0 from plx website		*/
/* ignore the configuration space registers						*/
/* we are only realy interested in the interrupt register- matt 2001			*/

#define PLX_LAS0_RANGE		0x00 /* local address space 0 range			*/
#define PLX_LAS1_RANGE		0x04 /* local address space 1 range			*/
#define PLX_LAS2_RANGE		0x08 /* local address space 2 range			*/
#define PLX_LAS3_RANGE		0x0c /* local address space 3 range			*/
#define PLX_EXP_ROM_RANGE	0x10 /* expansion rom range				*/
#define PLX_LAS0_LBA		0x14 /* local address space local base address		*/
#define PLX_LAS1_LBA		0x18 /* local address space local base address		*/
#define PLX_LAS2_LBA		0x1c /* local address space local base address		*/
#define PLX_LAS3_LBA		0x20 /* local address space local base address		*/
#define PLX_EXP_ROM_LBA		0x24 /* expansion rom local base address		*/
#define PLX_LAS0_BUS_DESCR	0x28 /* local space 0 region descriptor			*/
#define PLX_LAS1_BUS_DESCR	0x2c /* local space 1 region descriptor			*/
#define PLX_LAS2_BUS_DESCR	0x30 /* local space 2 region descriptor			*/
#define PLX_LAS3_BUS_DESCR	0x34 /* local space 3 region descriptor			*/
#define PLX_EXP_ROM_DESCR	0x38 /* expansion ROM region descriptor			*/
#define PLX_SEL_0_BASE		0x3c /* chip slect 0 base address ?			*/
#define PLX_SEL_1_BASE		0x40 /* chip slect 1 base address ?			*/
#define PLX_SEL_2_BASE		0x44 /* chip slect 2 base address ?			*/
#define PLX_SEL_3_BASE		0x48 /* chip slect 3 base address ?			*/
#define PLX_INTERRUPT		0x4c /* PLX interrupt control / status			*/
#define PLX_USERIO_PLUS		0x50 /* user io, slave resp., serial EEPROM & init cntrl*/


/*************************PLX INTERRUPT FLAGS *******************************************/
/* working from the PLX 9052 data book v 2.0 from plx website				*/
#define PLX_INT_LINT1_ON	0x0001	/* r/w local int 1 enabled, 1=on, 0=off		*/
#define PLX_INT_LINT1_POL	0x0002	/* r/w local int 1 polarity 1=active high, 0=low*/
#define PLX_INT_LINT1_STAT	0x0004	/* r   local int 1 status 1=active, 0=inactive	*/
#define PLX_INT_LINT2_ON	0x0008	/* r/w local int 2 enabled, 1=on, 0=off		*/
#define PLX_INT_LINT2_POL	0x0010	/* r/w local int 2 polarity 1=active high, 0=low*/
#define PLX_INT_LINT2_STAT	0x0020	/* r   local int 2 status 1=active, 0=inactive	*/
#define PLX_INT_PCI		0x0040	/* r/w PCI interrupt enable 1=enable		*/
#define PLX_INT_SOFT_INT	0x0080	/* r/w 1 generates int if PLX_INT_PCI enabled	*/
#define PLX_INT_LINT1_SEL	0x0100	/* r/w select edge(1) or level(0) trig on int 1	*/
#define PLX_INT_LINT2_SEL	0x0200	/* r/w select edge(1) or level(0) trig on int 2	*/
#define PLX_INT_LET_CLR1	0x0400	/* r/w clears edge triggered local int 1	*/
#define PLX_INT_LET_CLR2	0x0800	/* r/w clears edge triggered local int 2	*/
#define PLX_INT_ISA_MODE	0x1000	/* r   enables ISA interface mode (w->eeprom)	*/
/* bits 13:31 reserved */

/****************************************************************************************/
/* what the card looks like to the system interrupt wise				*/
/* Register PLX_INTERRUPT=63								*
 * PCI interrupts are enabled (soft off- good)						*
 * ISA mode is enabled									*
 * local int 1 is on, active high, level trig'd and inactive and ET unclear		*
 * local int 2 is off, active low, level trig'd and active and ET unclear		*
 * In practice this means probing for local int 1 active and resetting using 		*
 * PLX_INT_LET_CLR1									*/

/**************************** CARD REGISTER MAP *****************************************/
/* Register map working from the "512 Driver Address Map Doc from BNS 			*/

#define BNS_CONTROL	0x00	/* Control register 					*/
#define BNS_STATUS	0x04	/* Status register (read only) 				*/
#define BNS_IMAGE	0x08	/* Image number 					*/
#define BNS_PERIODS	0x0c	/* Image and TRUE frames 				*/
#define BNS_LASERA	0x10	/* Ouput (gain) and timing for output laser A		*/
#define BNS_LASERB	0x14	/* Ouput (gain) and timing for output laser B		*/
#define BNS_INTFRM	0x18	/* Interrupt frames (? matt)				*/

/* I think you read/write this next register (with anything) to change the vital
 * register in sync- matt 2001 */
#define	BNS_RRDY	0x1c	/* Registers Ready (Address only ?matt			*/
#define BNS_CLRBURP	0x20	/* Clear BURP condition ?matt 				*/
#define BNS_CLRINT	0x24	/* Clear interrupt (address only - significant matt)	*/
#define BNS_CURFRM	0x28	/* Current image (read only)				*/
#define BNS_TIMEID	0x2c	/* Location of timing and control FPGA (read only)	*/
#define BNS_SDRAM0	0x30	/* Location of SDRAM controller 0 FPGA (read only)	*/
#define BNS_SDRAM1	0x34	/* Location of SDRAM controller 0 FPGA (read only)	*/
#define BNS_SDRAM2	0x38	/* Location of SDRAM controller 0 FPGA (read only)	*/
#define BNS_SDRAM3	0x3c	/* Location of SDRAM controller 0 FPGA (read only)	*/

/****************************************************************************************/

/******************************** REGISTER BIT FIELDS ***********************************/
/* now the control bit fields */

/* enables transfer from Card memory to dual-port ram */

/* CNTL_DPEN enables data flow from card ram to port ram 				*
 * "SDRAM is not available to the PCI bus while transfer is in progress"		*
 * does this mean "the image that is being transferred" or				*
 *		  "all the image memory"						*
 * and does it mean "during transfer" or "while CNTL_DPEN enabled?			*
 *											*
 * I think set to 0 transfer host->pcicard, set to 1 transfer pci->port			*
 * set to 0 again before writing PCI- matt 						*
 * ie it is 0 host->pci	enable
 *	    1 pci->port enable
 */
/* Clarification- from Kipp 22 Oct 2001							*
 * It <should> be (by design) possible to write from host to (card) while		*
 * (card) writes to (port), however "10 percent of the time" this locks the 		*
 * computer (don't know if this on 10 percent of machines / 10 percent of 		*
 * driver installs of 10 percent of writes.  Anyway the recommendation is		*
 * disable (card)->(port) with DPEN=0 before any host to (card) mem access.
 * This "problem" will be fixed "someday"						*/
/* It turns out that BNS have not been probing the FIFO full buffers during DPEN open	*
 * writes- this is the reason for their problems- a back of the envelope calculation	*
 * indicates that these fifos are practically useless and having to probe the FULL	*
 * flags will actually SLOW the throughput- What we need is a flag that indicates when	*
 * 2port transfer is in progress so we can side step it- matt				*/

#define BNS_CNTL_DPEN		0x0001	/* Dual port transfer enable 			*/

/* DETA+DETB enable detector sync A+B- I think these are outputs but I am not sure	*/
#define BNS_CNTL_DETA		0x00000002	/* Detector sync A enable		*/
#define BNS_CNTL_DETB		0x00000004	/* Detecotr sync B enable 		*/

/* 1= enable card to gen interrupts 0= disbale						*/
#define BNS_CNTL_INTEN		0x00000008	/* Interrupt enable			*/

/* 1= enables registers ready ie vital registers are held until this port is addressed	*/
#define BNS_CNTL_RRDYEN		0x00000010	/* register ready enable		*/

/* Not entirely clear on this yet- it controls what is in control of the SDRAM		*
 * (card memory) bus, 0= SDRAM controllers (PCI card I think)				*
 * 1=External port internface (port) and inits the card->port transfer			*
 * a low here also clears the EPI_ERR bit in the status word 				*/
#define BNS_CNTL_EN_EXT_PORT	0x00000020	/* enable external port			*/

/* resets the (port) counters on the (card) side and (slm) side respectively		*/
#define BNS_CNTL_RST_2PORT_CNTR	0x00000040	/* reset dual ram counter ?matt		*/
#define BNS_CNTL_RST_SLM_CNTR	0x00000080	/* reset slm counter ?matt		*/

/* turns power onto the DAC, Opamps and SLM respectively				*
 * DAC raises lines DAC_PWR_CNTLA+B and the DAC board responds with 2PORT_INITd 	*
 * if it doesn't work DAC_xx are clear and DAC_PWR_ERR (STATUS) is raised       	*
 *
 * OPAMP enables the +OPAMP_CNTL to go high once 2PORT_INITd is high and 10ms   	*
 * later -OPAMP_CNTL goes high, they will go low if OPAMP or 2PORT goes low     	*
 *
 * SLM_PWR applies power to the SLM by raising SLM_PWR_CNTL the DAC board  		*
 * responds by raising ANALOG_PWR_OK, if this does not occur SLM_PWR is dropped 	*
 * (also dropped by ANALOG_PWR_ERR (STATUS)						*/
#define BNS_CNTL_DAC_PWR_CNTRL	0x00000100	/* enable DAC power on			*/
#define BNS_CNTL_OPAMP_PWR_CNTL	0x00000200	/* enable Opamp power on		*/
#define BNS_CNTL_SLM_PWR_CNTL	0x00000400	/* enable SLM power on			*/

/* a low on NEMATIC means ignore the TF field of PERIODS and use 1/2 true,      	*
 * 1/2 inverted for the image								*
 * a high here enables the user to program this with the TF field			*/
#define BNS_CNTL_NEMATIC		0x00000800	/* enable TF Field in PERIODS register	*/

/* this is a bit that enables FPGS reprogramming and should not be needed		*
 * ignore it- matt 2001 (but doe it need to be set ?)					*/
#define BNS_CNTL_SDRAM_PROG		0x00001000	/* reprogram sdram controllers ?matt	*/
/* bits 13-31 are unused */


/* STATUS register bit fields */

/* indicates if the SDRAM controllers are init'd (don't know if we need this-		*/
#define BNS_STAT_INIT0		0x00000001
#define BNS_STAT_INIT1		0x00000002
#define BNS_STAT_INIT2		0x00000004
#define BNS_STAT_INIT3		0x00000008

/* indicates SDRAM (card ram) fifo is full (again do we care??)				*/
/* in the event that (a) host->pci->card memory write AND 
 *		     (b) card memory->port memory 
 *		     ie DPEN is set
 * these bits MUST be polled to prevent missed writes (so writing to card
 * when card writes to port is legal???							*/
#define BNS_STAT_FULL0		0x00000010
#define BNS_STAT_FULL1		0x00000020
#define BNS_STAT_FULL2		0x00000040
#define BNS_STAT_FULL3		0x00000080
#define BNS_STAT_FULLX		(BNS_STAT_FULL0+BNS_STAT_FULL1+BNS_STAT_FULL2+BNS_STAT_FULL3)

/* indicates the status of the SLV/MST jumper on the card (ignore this)			*/
#define BNS_STAT_SLAVE		0x00000100

/* A high here indicates that the status of the gain compliment jumper (laser gains inverted) */
#define BNS_STAT_GAINCOMP		0x00000200

/* A diagnostic which indicates if the interrupt service routine is "fast enough"	*
 * to service all the register updates before a new "image period"			*/
#define BNS_STAT_RRDY_ERR		0x00000400

/* when the DAC board is powered up this goes high 					*/
#define BNS_STAT_2PORT_INIT		0x00000800

/* indicates the 3.3v status of the DAC board						*/
#define BNS_STAT_V_STATUS		0x00001000

/* goes high when SLM power ok is reached (also analog powr ok is reached)		*/
#define BNS_STAT_SLM_PWR_OK		0x00002000

/* indicates that 2PORT_INITd failed to go high and the (port) power is bad		*/
#define BNS_STAT_DAC_PWR_ERR	0x00004000

/* appears to be the inverse of SLM_PWR_OK 						*/
#define BNS_STAT_ANA_PWR_ERR	0x00008000

/* high here indicates dat transfer from (card)->(port) via EPI				*/
#define BNS_STAT_EPI_SEND		0x00010000

/* if the EPI is still sending at the begining of the next image period this goes high	*/
#define BNS_STAT_EPI_ERR		0x00020000

/* sets as a result of several PERIODS errors- indicates PERIODS has been over written
 * with safe DC balance values */
#define BNS_STAT_PERIODS_ERR	0x00040000

/* unused here for my counting purposes- matt*/
#define BNS_STAT_UNUSED		0x00080000

/* indicates the EPI "external port interface ID"					*
 * 0xf-> none present									*
 * 0x0-> EPI present but not init'd							*
 * 0x1->0xe is the ID for the init'd EPI						*
 * 0x1=Lockheed-Martin "modified Sundance SMT-335" 					*/
#define BNS_STAT_EPI_ID			0x00f00000 /* bits 20-23 */
#define BNS_STAT_EPI_SHIFT		20
#define BNS_GET_EPI_ID(a)		(BNS_STAT_EPI_ID&a)>>BNS_STAT_EPI_SHIFT
#define BNS_EPI_NOT_PRESENT		0xf
#define BNS_EPI_PRESENT_NOT_INIT	0x0
#define BNS_EPI_PRESENT_SUNDANCE	0x1

/* board memory ID byte, only bits 6+7 count at present,				*
 * 00=1024 image, 01=2048 images, 10=512 images, 11=undefined				*/
#define BNS_STAT_BOARD_ID		0xff00000 /* bits 24-31 */
#define BNS_STAT_BOARD_SHIFT		24
#define BNS_GET_MEM_BOARD_ID(a)	(BNS_STAT_BOARD_ID&a)>>BNS_STAT_BOARD_SHIFT
#define BNS_MEM_ID_MASK			0xc0
#define BNS_MEM_1024			0x00
#define BNS_MEM_2048			0x40
#define BNS_MEM_512			0x80
#define BNS_MEM_UNDEFINED		0xc0


/* IMAGE bit masks 									*/
/* the first 10 bits (0-9) refer to the image to transfer to the (port)			*/
/* the next size bits are unused							*
 * the next 10 bits (16-25) refere to the image that the PCI area writes 		*/
/* the next 6 are unused */
#define BNS_DP_IMAGE		0x000003ff
#define BNS_MEM_IMAGE		0x03ff0000

/* PERIODS register indicating the number of TRUE and INVERTED frames			*/
/* the image periods indicates how many frame periods the image is shown		*
 * the true period indicates how long it is TRUE, 					*
 * In order to acheive DC balance the lsb of image periods is dropped			*
 * image periods has a hard minimum of 6 (enforced by hardware				*
 * if NEMATIC is 1 then image periods must be an integer multiple of (true		*
 * periods * 2)										*
 * true periods is automagically set if NEMATIC is 0					*
 * otherwise it should be set so that (2*T_P)*N=I_P and the inversion will take		*
 * place more often									*/ 
#define BNS_IMAGE_PERIODS		0x0000ffff	/* I_P */
#define BNS_TRUE_PERIODS		0x7fff0000	/* T_P */


/* LASER GAINS -same for laser a and b 							*/
/* gain is an 8 bit number giving 0-5v in 256 steps on the analogue outs		*
 * fine BNS_delay is 1/16s of a frame period from "load" to "laser on"			*
 * coarse delay is # frame period from the start of the image period to laser on	*/
#define BNS_LASER_GAIN		0x000000ff
#define BNS_LASER_FINE_DELAY	0x00000f00
#define BNS_LASER_COARSE_DELAY	0xffff0000
#define BNS_DEF_DELAY		0x00040000

/* INTFRM Interrupt frames register 							*/
/* the number of frames from the begining of the image period to the interrupt		*
 * interrupts only occur if enabled with INTEN bit					*/
#define BNS_INTFRM_MASK		0x0000ffff

/* RRDY register ready word */
/* write anything to this to load the sensitive registers together			*
 * if there is an error (ie registers not written before (port) load			*
 * then the RRDY_ERR bit is set								*/
/* CLRBURP write this anything to clear the RRDY_ERR bit				*/
/* CLRINT write this to clear interrupts (done by hardware each frame !			*/
/* CURFRM read this for the # frames remaining before the end of the image period	*/
/* timingID read to get the timing ID for the FPGA program version number		*/
/* SDRAMxID read these to get the SDRAM controller FGPA program IDs			*/

/* reading BNS_RAMx registers returns the following, note _OK is revision dependent	*/
#define BNS_SDRAM_FAIL	0xffffffff
/*#define BNS_SDRAM_OK	0x00000009  old board */
#define BNS_SDRAM_OK	0x0000000b /* new board */


/****************************************************************************************/
/* INIT'ing contants- time outs etc							*/

/* that is 200ms usual wait divided by 10ms (loop sleep time) * 5 for good measure	*/
#define BNS_PWR_TIMEOUT_LOOPS	(200/10)*5	
#define BNS_PWR_ERR		-1


/****************************************************************************************/
/* private data types									*/


/* Now a design decision had to be made, static array, dynamic array or linked list?	*
 * and when to allocate the frames? init or open- since we will allow it to be open	*
 * no more than once at a time only and I want the card to run even when the device	*
 * is not open (so scripts can use it) the irl has to be static and alloc'd at init time* 
 * I choose dynamic array allocated at init time for simplicity but it is likely to 	*
 * be wasteful of a couple of memory pages for most use but it was easier than the 	*
 * linked list in implementation							*/

/* following this fashion the slm will be represented by a static structure		*/
/* This will make it easier to include additional cards in future editions		*/

//
struct slm_t{
/* divide this up									*/
/* slm board parameters									*/
	unsigned long		phys_mem;	/* physical address of mem for mtrr 	*/
	void *			mem;		/* our memory window			*/
	void *			io;		/* our io space				*/
	void *			plx_chip;	/* plx card controller regs		*/
	int 			interrupt;	/* our interrupt #			*/
	int			max_images;	/* no image slots in memory		*/
	int			sdram_id;	/* id of the sdram controllers		*/
	int			epi_id;		/* id of external port			*/
	int			mem_board_id;	/* id of memory board			*/

/* software operation parameters							*/
/* image rotation list params								*/
	int			list_pos;	/* where we are in the list		*/
	int			n_list;		/* no images in the list		*/
	BNS_irl_object 		* image_list;	/* pointer to list memory		*/

/* operational parms									*/
	int			DPEN_mode;	/* how to side step DPEN		*/
	int			DPEN_enable;	/* DPEN status				*/
	int 			DPEN_delay;	/* the DPEN delay microseconds		*/
	int			image_mode;	/* SLM_OFF, _LIST or _IMAGE		*/
	int			cur_image;	/* the current image for mode _IMAGE	*/
	int			periods;	/* periods # for a cur_image		*/
	int			mem_image;	/* current upload image			*/
	int			power_status; 	/* power status?- may change-matt	*/
	int			row;		/* current upload row			*/
	int			transfer_mode;	/* set to PERFORM_SCRAMBLE		*/
	int			locked;		/* per board locking			*/
	int			lasera,laserb;	/* make them global			*/

/* diagnostics										*/
	int			mem_errs;	/* monitor memory error occurance	*/
	int			mem_err0x89;	/* monitor memory error occurance	*/
	int			int_errs;	/* monitor int error occurance		*/
	int			ints;		/* monitor # of interrupts		*/
	int			not_ints;	/* monitor # of ints that aren't ours	*/
	int			uploaded;	/* total no of images uploaded		*/
	int			displayed;	/* total no of images displayed		*/
#ifdef BNS_PROFILE
	int			sec_copy_from;	/* profiling numbers			*/
	int			sec_scramble;	/* profiling numbers			*/
	int			sec_pci_write;	/* profiling numbers			*/
#endif
	};

/* time for some defaults								*/
#define LASERA_DEF	0x000000ff | BNS_DEF_DELAY
#define LASERB_DEF	0x000000ff | BNS_DEF_DELAY
#define PERIODS_DEF	0x0000ffff
#define NAV	-1	/* not a value	*/

static struct slm_t slm={	
	phys_mem:	0,
	mem:		NULL,
	io:		NULL,
	plx_chip:	NULL,
	interrupt:	NAV,
	max_images:	NAV,
	sdram_id:	NAV,
	epi_id:		NAV,
	mem_board_id:	NAV,
	list_pos:	NAV,
	n_list:		NAV,
	image_list:	NULL,
	DPEN_mode:	BNS_DPEN_MODE_BUSY,
	DPEN_enable:	FALSE,
	DPEN_delay:	BNS_DEFAULT_DPEN_DELAY,
	image_mode:	BNS_SLM_OFF,
	cur_image:	NAV,
	periods:	PERIODS_DEF,
	mem_image:	0,
	power_status:	BNS_POWER_OFF,
	row:		0,
	transfer_mode:	PERFORM_SCRAMBLE,	
	locked:		FALSE,
	lasera:		LASERA_DEF,
	laserb:		LASERB_DEF,
	mem_errs:	0,
	mem_err0x89:	0,
	int_errs:	0,
	ints:		0,
	not_ints:	0,
	uploaded:	0,
	displayed:	0,
#ifdef BNS_PROFILE
	sec_copy_from:	0,
	sec_scramble:	0,
	sec_pci_write:	0,
#endif
	};


MODULE_LICENSE("GPL");

/* end the section for driver only code **************************************************/ 
#endif
/******** THIS IS THE END OF THE HEADER CODE *********************************************/
